<?xml version="1.0"?>
<xsl:stylesheet
  version='1.0'
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:math="http://exslt.org/math"
  extension-element-prefixes="exslt math">

  <!-- These variables are going to be queried in the context of a generated node set. -->
  <xsl:variable name="report" select="/"/>
  
  <!-- Major issues. -->
  <xsl:variable name="majorIssueNodes">
    
    <!-- No disks found. -->
    <value name="nodisks">
      <xsl:value-of select="count($report/etrecheck/disk/drive) = 0"/>
    </value>
    
    <!-- No volumes found. -->
    <value name="novolumes">
      <xsl:value-of
        select="count($report/etrecheck/virtualvolume/volume) = 0"/>
    </value>
    
    <!-- No uptime found. -->
    <value name="nouptime">
      <xsl:value-of select="not($report/etrecheck/systemsoftware/uptime)"/>
    </value>
    
    <!-- No operating system build version found. -->
    <value name="noosbuild">
      <xsl:value-of select="not($report/etrecheck/systemsoftware/build)"/>
    </value>
    
    <!-- Time Machine not configured. -->
    <value name="tmnotconfigured">
      <xsl:variable name="notconfigured">
        <xsl:call-template name="checktmnotconfigured"/>
      </xsl:variable>
      
      <xsl:value-of select="$notconfigured = 'true'"/>
    </value>

    <!-- Time Machine out of date. -->
    <value name="tmold">
      <xsl:call-template name="checktmold"/>
    </value>

    <!-- Hardware failure. -->
    <value name="hardwarefailure">
      <!-- <xsl:call-template name="checkhardwarefailure"/> -->
      <xsl:value-of select="false()"/>
    </value>

    <!-- Battery failure. -->
    <value name="batteryfailure">
      <xsl:value-of
        select="$report/etrecheck/hardware/batteryhealth = 'Check Battery'"/>
    </value>

    <!-- SMART status failure. -->
    <value name="smartfailure">
      <xsl:value-of
        select="$report/etrecheck/disk//smartstatus = 'Failing'"/>
    </value>
  
    <!-- RAM status failure. -->
    <value name="ramfailure">
      <xsl:value-of
        select="count($report/etrecheck/hardware/memorybanks/memorybank[status != 'ok']) > 0"/>
    </value>

    <!-- Disk errors. -->
    <value name="diskerrors">
      <xsl:value-of
        select="$report/etrecheck/disk//errors/error or $report/etrecheck/virtualvolume//errors/error"/>
    </value>
  
    <!-- Hard drive failure. -->
    <value name="hddfailure">
      <xsl:call-template name="checkdrivefailure"/>
    </value>
  
    <!-- Hard drive too slow. -->
    <value name="hddslow">
      <xsl:call-template name="checkhddslow"/>
    </value>

    <!-- Runaway kernel_task. -->
    <value name="runawaykernel">
      <xsl:value-of
        select="sum($report/etrecheck/cpu/process[path = 'kernel_task']/usage) &gt; 150"/>
    </value>
  
    <!-- Runaway process. -->
    <value name="runawayprocess">
      <xsl:variable name="icloudprocs">
        <xsl:call-template name="checkicloudprocs"/>
      </xsl:variable>
      
      <xsl:variable name="photosprocs">
        <xsl:call-template name="checkphotosprocs"/>
      </xsl:variable>

      <xsl:variable
        name="usagecount"
        select="count($report/etrecheck/cpu/process[other = 'true' and usage &gt; 80])"/>

      <xsl:choose>
        <xsl:when test="$icloudprocs = 'true'">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:when test="$photosprocs = 'true'">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$usagecount &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>

    <!-- Proxies. -->
    <value name="proxies">
      <xsl:value-of
        select="count($report/etrecheck//networkinterface//proxy) &gt; 0"/>
    </value>
  
    <!-- Adware. -->
    <value name="adware">
      <xsl:variable
        name="launchdcount"
        select="count($report/etrecheck/adware/launchdfiles/launchdfile)"/>
        
      <xsl:variable
        name="loginitemcount"
        select="count($report/etrecheck/adware/loginitems/loginitem)"/>

      <xsl:variable
        name="adwarecount"
        select="$launchdcount + $loginitemcount"/>

      <xsl:value-of select="$adwarecount &gt; 0"/>
    </value>
  
    <!-- Probable adware. -->
    <value name="majorunsigned">
      <xsl:call-template name="checkmajorunsigned"/>
    </value>

    <!-- Gatekeeper disabled. -->
    <value name="gatekeeperdisabled">
      <xsl:call-template name="checkgatekeeperdisabled"/>
    </value>

    <!-- SIP disabled. -->
    <value name="sipdisabled">
      <xsl:call-template name="checksipdisabled"/>
    </value>

    <!-- Automatic updates disabled. -->
    <value name="automaticupdatesdisabled">
      <xsl:value-of
        select="$report/etrecheck/security/softwareupdate/automaticupdatesdisabled = 'true'"/>
    </value>

    <!-- Critical updates disabled. -->
    <value name="criticalupdatesdisabled">
      <xsl:value-of
        select="$report/etrecheck/security/softwareupdate/criticalupdatesdisabled = 'true'"/>
    </value>

    <!-- Old operating system. -->
    <value name="oldos">
      <xsl:value-of
        select="$report/etrecheck/systemsoftware/majorversion &lt; 13"/>
    </value>
  
    <!-- Kernel panics. -->
    <value name="kernelpanics">
      <xsl:variable
        name="kernelpaniccount"
        select="sum($report/etrecheck/diagnostics/event[(type = 6 or type = 14) and string-length(information) &gt; 0]/count) &gt; 0"/>

      <xsl:variable
        name="applekernelpaniccount"
        select="sum($report/etrecheck/diagnostics/event[(type = 6 or type = 14) and string-length(information) = 0]/count)"/>
        
      <xsl:variable
        name="bridgepaniccount"
        select="sum($report/etrecheck/diagnostics/event[type = 12]/count)"/>

      <xsl:value-of select="$kernelpaniccount &gt; 0 and ($applekernelpaniccount + $bridgepaniccount) = 0"/>
    </value>
  
    <!-- Kernel panics with no 3rd party extensions. -->
    <value name="applekernelpanics">
      <xsl:variable
        name="applekernelpaniccount"
        select="sum($report/etrecheck/diagnostics/event[(type = 6 or type = 14) and string-length(information) = 0]/count)"/>
        
      <xsl:variable
        name="bridgepaniccount"
        select="sum($report/etrecheck/diagnostics/event[type = 12]/count)"/>

      <xsl:value-of select="($applekernelpaniccount + $bridgepaniccount) &gt; 0"/>
    </value>

    <!-- Poor performance without a hdd explanation. -->
    <value name="poorperformance">
      <xsl:call-template name="checkpoorperformance"/>
    </value>
  
    <!-- Heavy CPU usage from a normal process. -->
    <value name="heavycpu">
      <xsl:variable name="icloudprocs">
        <xsl:call-template name="checkicloudprocs"/>
      </xsl:variable>
      
      <xsl:variable name="photosprocs">
        <xsl:call-template name="checkphotosprocs"/>
      </xsl:variable>

      <xsl:choose>
        <xsl:when test="$icloudprocs = 'true'">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:when test="$photosprocs = 'true'">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="sum($report/etrecheck/cpu/process[path != 'kernel_task' and other != 'true']/usage) &gt; 150"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>
  
    <!-- Critically low hard drive space. -->
    <value name="majorlowdiskspace">
      <xsl:call-template name="checkmajorlowdiskspace"/>
    </value>

    <!-- No AV installed. -->
    <value name="noav">
      <xsl:choose>
        <xsl:when test="not($report/etrecheck/security/appleav)">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$report/etrecheck/security/appleav = 'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>
    
    <!-- Critically low RAM. -->
    <value name="lowram">
      
      <xsl:variable name="k">1024</xsl:variable>
      
      <xsl:variable name="number" select="$report/etrecheck/hardware/ram"/>
      
      <xsl:variable name="ram">
        <xsl:choose>
          <xsl:when test="$number/@units = 'PB'">
            <xsl:value-of select="$number * $k * $k * $k"/>
          </xsl:when>
          <xsl:when test="$number/@units = 'TB'">
            <xsl:value-of select="$number * $k * $k"/>
          </xsl:when>
          <xsl:when test="$number/@units = 'GB'">
            <xsl:value-of select="$number * $k"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>1024</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:value-of select="$ram &lt; 4096"/>
    </value>
  
    <!-- Obsolete machine. -->
    <value name="obsolete">
      <xsl:value-of
        select="$report/etrecheck/hardware/vintage = 'obsolete'"/>
    </value>

    <!-- Safe mode. -->
    <value name="safemode">
      <xsl:value-of
        select="$report/etrecheck/systemsoftware/bootmode = 'safe_boot'"/>
    </value>

    <!-- Orphan system extension. -->
    <value name="orphansystemextension">
      <xsl:value-of
        select="count($report/etrecheck/cleanup/systemextensions/systemextension[orphan = 'true' and status != 'terminated_waiting_to_uninstall_on_reboot']) &gt; 0"/>
    </value>

    <!-- Blocked system extension. -->
    <value name="systemextensionblocked">
      <xsl:value-of
        select="count($report/etrecheck/systemextensions/systemextension[status = 'activated_waiting_for_user']) &gt; 0"/>
    </value>

    <!-- Blocked kernel extensions. -->
    <value name="extensionsblocked">
      <xsl:value-of
        select="count($report/etrecheck/kernelextensions//extension[status = 'blocked']) &gt; 0"/>
    </value>
  
    <value name="flashinstalled">
      <xsl:value-of select="$report/etrecheck/header/flashinstalled"/>
    </value>

  </xsl:variable>
  
  <!-- Minor issues. -->
  <xsl:variable name="minorIssueNodes">
    
    <!-- SSD failure. -->
    <value name="ssdfailure">
      <xsl:call-template name="checkssdfailure"/>
    </value>
    
    <!-- Multiple antivirus apps installed. -->
    <value name="multipleav">
      <xsl:value-of select="$report/etrecheck/adware/avcount3rdparty &gt; 1"/>
    </value>
  
    <!-- System extensions installed. -->
    <value name="systemextensions">
      <xsl:value-of select="count($report/etrecheck/systemextensions/systemextension[status = 'activated_enabled']) &gt;= 1"/>
    </value>

    <!-- Time Machine not configured. -->
    <value name="minortmnotconfigured">
      <xsl:call-template name="checkminortmnotconfigured"/>
    </value>

    <!-- Time Machine disabled. -->
    <value name="tmdisabled">
      <xsl:call-template name="checktmdisabled"/>
    </value>

    <!-- Stuck iCloud. -->
    <value name="icloudstuck">
      <xsl:value-of
        select="$report/etrecheck/network/icloudpendingfiles &gt; 10"/>
    </value>
  
    <!-- Shutdown due to overheating. -->
    <value name="overheating">
      <!-- <xsl:call-template name="checkoverheating"/> -->
      <xsl:value-of select="false()"/>
    </value>

    <!-- Modified sudoers file. -->
    <value name="modifiedsudoers">
      <xsl:value-of
        select="$report/etrecheck/configurationfiles/unexpectedsudoerssize"/>
    </value>
  
    <!-- Configuration profiles present. -->
    <value name="configurationprofiles">
      <xsl:value-of
        select="count($report/etrecheck/configurationprofiles/configurationprofile/setting) &gt; 0 or $report/etrecheck/configurationprofiles/hasconfigurationprofiles = 'true'"/>
    </value>
  
    <!-- Chrome settings present. -->
    <value name="chromedefaults">
      <xsl:value-of
        select="count($report/etrecheck/configurationprofiles/chrome/setting) &gt; 0"/>
    </value>

    <!-- Low disk space. -->
    <value name="minorlowdiskspace">
      <xsl:call-template name="checkminorlowdiskspace"/>
    </value>

    <!-- Core files. -->
    <value name="corefiles">
      <xsl:value-of
        select="$report/etrecheck/configurationfiles/corecount &gt; 0"/>
    </value>

    <!-- iCloud processes. -->
    <value name="icloudprocs">
      <xsl:call-template name="checkicloudprocs"/>
    </value>

    <!-- Photos processes. -->
    <value name="photosprocs">
      <xsl:call-template name="checkphotosprocs"/>
    </value>

    <!-- Upgradeable hard drive. -->
    <value name="upgradeablehd">
      <xsl:call-template name="checkupgradeablehd"/>
    </value>

    <!-- Upgradeable RAM. -->
    <value name="upgradeableram">
      <xsl:choose>
        <xsl:when test="$report/etrecheck/hardware/ram &gt;= 8">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:when test="$report/etrecheck/hardware/userupgradeable = 'false'">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:when test="$report/etrecheck/hardware/applemaximummemory &gt;= $report/etrecheck/hardware/ram">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:when test="$report/etrecheck/hardware/upgradeable = 'true'">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>

    <!-- Heavy RAM usage. -->
    <value name="heavyram">
      <xsl:call-template name="checkheavyram"/>
    </value>

    <!-- Heavy network usage. -->
    <value name="heavynetwork">
      <xsl:call-template name="checkheavynetwork"/>
    </value>

    <!-- Out-of-RAM conditions with a mechanical hard drive. -->
    <value name="outofram">
      <xsl:call-template name="checkoutofram"/>
    </value>

    <!-- Older machine with a new operating system. -->
    <value name="oldmachinenewos">
      <xsl:call-template name="checkoldmachinenewos"/>
    </value>

    <!-- Time Machine drive is too small. -->
    <value name="tmtoosmall">
      <xsl:call-template name="checktmtoosmall"/>
    </value>

    <!-- High battery cycles. -->
    <value name="highbatterycycles">
      <xsl:value-of
        select="$report/etrecheck/hardware/batterypercent &gt; 70"/>
    </value>
  
    <!-- Shutdown due to corrupt file system. -->
    <value name="corruptfilesystemshutdown">
      <!-- <xsl:value-of
        select="sum($report/etrecheck/diagnostics/event[type = 10 and code = -60]/count) &gt; 0"/> -->
      <xsl:value-of select="false()"/>
    </value>
  
    <!-- Apps are crashing. -->
    <value name="appscrashing">
      <xsl:value-of
        select="sum($report/etrecheck/diagnostics/event[type = 1]/count) &gt; 5"/>
    </value>
  
    <!-- Apps are hanging. -->
    <value name="appshanging">
      <xsl:value-of
        select="sum($report/etrecheck/diagnostics/event[type = 3]/count) &gt; 5"/>
    </value>
  
    <!-- Apps are reporting CPU usage. -->
    <value name="appscpu">
      <xsl:value-of
        select="sum($report/etrecheck/diagnostics/event[type = 2]/count) &gt; 5"/>
    </value>
  
    <!-- Clean up opportuntites. -->
    <value name="cleanup">
      <xsl:choose>
        <xsl:when test="count($report/etrecheck/cleanup//launchdfile) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="count($report/etrecheck/cleanup//extension) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="count($report/etrecheck/cleanup//systemextension[status = 'terminated_waiting_to_uninstall_on_reboot']) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>
  
    <!-- Unsigned files that are probably OK. -->
    <value name="minorunsigned">
      <xsl:call-template name="checkminorunsigned"/>
    </value>
  
    <!-- Large number of system modifications. -->
    <value name="systemmodifications">
      <xsl:value-of
        select="count($report/etrecheck/launchagents/launchdfile) &gt; 10 or count($report/etrecheck/launchdaemons/launchdfile) &gt; 10"/>
    </value>
  
    <!-- Runaway process. -->
    <value name="minorrunawayprocess">
      <xsl:variable name="icloudprocs">
        <xsl:call-template name="checkicloudprocs"/>
      </xsl:variable>
      
      <xsl:variable name="photosprocs">
        <xsl:call-template name="checkphotosprocs"/>
      </xsl:variable>

      <xsl:variable
        name="usagecount"
        select="count($report/etrecheck/cpu/process[other != 'true' and usage &gt; 80])"/>

      <xsl:choose>
        <xsl:when test="$icloudprocs = 'true'">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:when test="$photosprocs = 'true'">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$usagecount &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>

    <!-- Below average performance. -->
    <value name="belowaverageperformance">
      <xsl:call-template name="checkbelowaverageperformance"/>
    </value>
  
    <!-- Corrupt hosts file. -->
    <value name="corrupthosts">
      <xsl:value-of
        select="$report/etrecheck/configurationfiles/hostscorrupt = 'true'"/>
    </value>
  
    <!-- Vintage machine. -->
    <value name="vintage">
      <xsl:value-of
        select="$report/etrecheck/hardware/vintage = 'vintage'"/>
    </value>
  
    <!-- Encryption in progress. -->
    <value name="encrypting">
      <xsl:value-of
        select="$report/etrecheck//volume/encryption/status = 'encrypting'"/>
    </value>
  
    <!-- Decryption in progress. -->
    <value name="decrypting">
      <xsl:value-of
        select="$report/etrecheck//volume/encryption/status = 'decrypting'"/>
    </value>
  
    <!-- Encryption or decryption in progress. -->
    <value name="inprogress">
      <xsl:value-of
        select="$report/etrecheck//volume/encryption/status = 'inprogress'"/>
    </value>

    <!-- High nominal I/O. -->
    <value name="nominalio">
      <xsl:value-of
        select="$report/etrecheck/performance/nominalio &gt; 20"/>
    </value>

    <!-- Insufficient permissions. -->
    <value name="insufficientpermissions">
      <xsl:value-of
        select="$report/etrecheck/diagnostics/insufficientpermissions = 'true'"/>
    </value>
  
    <!-- 32-bit apps. -->
    <value name="has32bitapps">
      <xsl:choose>
        <xsl:when
          test="$report/etrecheck/systemsoftware/majorversion &gt;= 15">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of
          select="count($report/etrecheck/applications/application[has64bit = 'false']) &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>
  
    <!-- Intel apps. -->
    <value name="hasintelapps">
      <xsl:choose>
        <xsl:when
          test="$report/etrecheck/systemsoftware/majorversion &lt; 15">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of
          select="count($report/etrecheck/applications/application[hasarm64 = 'false']) &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </value>

    <!-- Abnormal shutdown. -->
    <value name="shutdown">
      <!-- <xsl:call-template name="checkshutdown"/> -->
      <xsl:value-of select="false()"/>
    </value>

    <!-- USB 3.0 drives with USB 2.0 hub. -->
    <value name="usb2hub">
      <xsl:call-template name="checkusb2hub"/>
    </value>

    <!-- External drives and poor performance. -->
    <value name="externaldriveperformance">
      <xsl:call-template name="checkexternaldriveperformance"/>
    </value>

    <!-- Drive access limited. -->
    <value name="driveaccesslimited">
      <xsl:value-of select="$report/etrecheck/header/fulldriveaccess = 'false'"/>
    </value>

    <!-- Kernel boot args. -->
    <value name="kernelargs">
      <xsl:value-of
        select="string-length($report/etrecheck/systemsoftware/bootargs) &gt; 0"/>
    </value>

    <!-- Kernel extensions installed. -->
    <value name="kernelextensions">
      <xsl:value-of
        select="count($report/etrecheck/kernelextensions//extension) &gt; 0"/>
    </value>

    <!-- Sharing services enabled. -->
    <value name="sharing">
      <xsl:value-of
        select="count($report/etrecheck/network/sharing/service) &gt; 0"/>
    </value>

  </xsl:variable>
  
  <!-- Now convert to a nodeset for performing xpath queries. -->
  <xsl:variable
    name="sourceMajorIssues" select="exslt:node-set($majorIssueNodes)"/>

  <!-- Now convert to a nodeset for performing xpath queries. -->
  <xsl:variable
    name="sourceMinorIssues" select="exslt:node-set($minorIssueNodes)"/>

  <!-- Now convert to a nodeset for performing xpath queries. -->
  <xsl:variable
    name="issueOverrides"
    select="$report/etrecheck/overrides/issues"/>

  <!-- Apply overrides. -->
  <xsl:variable name="sourceMajorIssueNodes">
    
    <xsl:for-each select="$sourceMajorIssues/value">
      <xsl:variable name="name" select="@name"/>
      
      <xsl:choose>
        <xsl:when test="$issueOverrides/value[@name = $name]">
          <xsl:copy-of select="$issueOverrides/value[@name = $name]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="."/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
      
  </xsl:variable>

  <xsl:variable name="sourceMinorIssueNodes">
    
    <xsl:for-each select="$sourceMinorIssues/value">
      <xsl:variable name="name" select="@name"/>
      
      <xsl:choose>
        <xsl:when test="$issueOverrides/value[@name = $name]">
          <xsl:copy-of select="$issueOverrides/value[@name = $name]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="."/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
      
  </xsl:variable>

  <!-- Now convert to a nodeset for performing xpath queries. -->
  <xsl:variable
    name="majorIssues" select="exslt:node-set($sourceMajorIssueNodes)"/>

  <!-- Now convert to a nodeset for performing xpath queries. -->
  <xsl:variable
    name="minorIssues" select="exslt:node-set($sourceMinorIssueNodes)"/>

  <!-- Major issues. -->
  <xsl:template name="majorissues">
    
    <xsl:for-each select="$majorIssues/value[. = 'true']">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">
          <xsl:value-of select="@name"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:for-each>

  </xsl:template>

  <!-- Minor issues. -->
  <xsl:template name="minorissues">

    <xsl:for-each select="$minorIssues/value[. = 'true']">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">
          <xsl:value-of select="@name"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:for-each>

  </xsl:template>

  <!-- Check for Time Machine not configured. -->
  <xsl:template name="checktmnotconfigured">
  
    <xsl:variable name="other">
      <xsl:choose>
        <xsl:when
          test="count($report/etrecheck//launchdfile[contains(path, 'bombich')]) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when
          test="count($report/etrecheck//launchdfile[contains(path, 'shirtpocket')]) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$report/etrecheck/timemachine/notconfigured = 'true'">
        <xsl:value-of
          select="$other = 'false' and $fulldriveaccess = 'true'"/>
      </xsl:when>
      <xsl:when
        test="count($report/etrecheck/timemachine/destinations) = 0">
        <xsl:value-of
          select="$other = 'false' and $fulldriveaccess = 'true'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for Minor Time Machine not configured. -->
  <xsl:template name="checkminortmnotconfigured">
  
    <xsl:variable name="other">
      <xsl:choose>
        <xsl:when
          test="count($report/etrecheck//launchdfile[contains(path, 'bombich')]) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when
          test="count($report/etrecheck//launchdfile[contains(path, 'shirtpocket')]) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$report/etrecheck/timemachine/notconfigured = 'true'">
        <xsl:value-of
          select="$other = 'true' or $fulldriveaccess != 'true'"/>
      </xsl:when>
      <xsl:when
        test="count($report/etrecheck/timemachine/destinations) = 0">
        <xsl:value-of
          select="$other = 'true' or $fulldriveaccess != 'true'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for old Time Machine. -->
  <xsl:template name="checktmold">
  
    <xsl:choose>
      <xsl:when test="$fulldriveaccess != 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$report/etrecheck/timemachine/notconfigured = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="sum($report/etrecheck/timemachine//count) = 0">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when
        test="count($report/etrecheck/timemachine//destination/lastbackupold[. = 'true']) &gt; 0 and count(/etrecheck/timemachine//destination[not(lastbackupold) and lastbackup]) = 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for hardware failure. -->
  <xsl:template name="checkhardwarefailure">
  
    <xsl:choose>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -75]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -78]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -79]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -103]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for Unsigned files that don't look safe. -->
  <xsl:template name="checkmajorunsigned">
  
    <xsl:variable
      name="unsafecount"
      select="count($report/etrecheck/unsigned//launchdfile[apple != 'true' and (not(details) or (details != 'whitelist' and details != 'whitelistprefix' and details != 'signatureoverride' and details != 'legitimate' and details != 'legitimate_prefix' and details != 'executablemaspermissions' and details != 'plistpermissions' and details != 'executablepermissions'))])"/>
    
    <xsl:choose>
      <xsl:when test="$hideunsignedfiles = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$unsafecount &gt; 0"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Check for Gatekeeper disabled. -->
  <xsl:template name="checkgatekeeperdisabled">

    <xsl:choose>
      <xsl:when
        test="$report/etrecheck/security/gatekeeper/status = 'Mac App Store and identified developers'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when
        test="$report/etrecheck/security/gatekeeper/status = 'Mac App Store'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when
        test="$report/etrecheck/security/gatekeeper/status = 'App Store and identified developers'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when
        test="$report/etrecheck/security/gatekeeper/status = 'App Store'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when
        test="$report/etrecheck/security/gatekeeper/status = 'Enabled'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="true()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for SIP disabled. -->
  <xsl:template name="checksipdisabled">
    
    <xsl:choose>
      <xsl:when test="$report/etrecheck/security/SIP = 'enabled'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when
        test="$report/etrecheck/systemsoftware/majorversion &lt; 11">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="true()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Check iCloud procs. -->
  <xsl:template name="checkicloudprocs">

    <xsl:variable
      name="icloudprocs"
      select="$report/etrecheck/cpu/process[name = 'bird' or name = 'cloudd']"/>
      
    <xsl:variable
      name="problems"
      select="$icloudprocs[usage &gt; 50]"/>
      
    <xsl:value-of select="count($problems) &gt; 0"/>
    
  </xsl:template>

  <!-- Check Photos procs. -->
  <xsl:template name="checkphotosprocs">

    <xsl:variable
      name="photosprocs"
      select="$report/etrecheck/cpu/process[name = 'photoanalysisd' or name = 'photolibraryd' or name = 'com.apple.photomoments' or name = ' com.apple.photos.ImageConversionService']"/>
      
    <xsl:variable
      name="problems"
      select="$photosprocs[usage &gt; 50]"/>
      
    <xsl:value-of select="count($problems) &gt; 0"/>
    
  </xsl:template>

  <!-- Check for major low disk space. -->
  <xsl:template name="checkmajorlowdiskspace">

    <!-- Low disk space. < 20 GB -->
    <xsl:variable name="freespace">
      <xsl:call-template name="expand">
        <xsl:with-param
          name="number"
          select="$report/etrecheck//volume[mountpoint = '/']/free"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:choose>
      <xsl:when test="$freespace &lt; 20000000000">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>
  
  <!-- Check for overheating. -->
  <xsl:template name="checkoverheating">

    <xsl:choose>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -3]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -86]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -71]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -74]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -74]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -95]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -100]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Check for Time Machine disabled. -->
  <xsl:template name="checktmdisabled">

    <xsl:choose>
      <xsl:when test="$fulldriveaccess != 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when
        test="$report/etrecheck/timemachine/notconfigured = 'true' or sum($report/etrecheck/timemachine//count) = 0">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$report/etrecheck/timemachine/autobackup != 'true'">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Check for abnormal shutdown. -->
  <xsl:template name="checkshutdown">

    <xsl:choose>
      <!-- Hardware failures. -->
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -75]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -78]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -79]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -103]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -3]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <!-- Overheating. -->
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -86]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -71]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -74]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -95]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = -100]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <!-- Normal. -->
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10 and code = 5]/count) &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>
    
      <!-- Something else. -->
      <xsl:when
        test="sum($report/etrecheck/diagnostics/event[type = 10]/count) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      
      <!-- No shutdown code at all. -->
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Check for minor low disk space. -->
  <xsl:template name="checkminorlowdiskspace">
  
    <xsl:variable name="freespace">
      <xsl:call-template name="expand">
        <xsl:with-param
          name="number"
          select="$report/etrecheck//volume[mountpoint = '/']/free"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:choose>
      <xsl:when
        test="$freespace &gt;= 20000000000 and $freespace &lt; 30000000000">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>
  
  <!-- Upgradeable hard drive. -->
  <xsl:template name="checkupgradeablehd">
  
    <xsl:choose>
      <xsl:when
        test="//drive[boot = 'true' and mechanism = 'mechanical'] and $report/etrecheck/hardware/modeltype = 'MacBookPro'">
        <xsl:value-of select="true()"/>
      </xsl:when>
      
      <xsl:when
        test="//drive[boot = 'true' and mechanism = 'mechanical'] and $report/etrecheck/hardware/modeltype = 'MacBook'">
        <xsl:value-of select="true()"/>
      </xsl:when>

      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Underpowered machine. -->
  <xsl:template name="checkunderpowered">
  
    <xsl:variable
      name="bootdisk" select="$report/etrecheck/disk/drive[boot = 'true']"/>
    
    <xsl:choose>
      <xsl:when
        test="$bootdisk[mechanism = 'mechanical'] and $report/etrecheck/hardware/ram &lt; 8 and $report/etrecheck/hardware/upgradeable = 'false'">
        <xsl:value-of select="true()"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for heavy ram usage. -->
  <xsl:template name="checkheavyram">
  
    <xsl:variable
      name="kernel_task"
      select="$report/etrecheck/memory/process[path = 'kernel_task']"/>
    
    <xsl:variable
      name="other"
      select="$report/etrecheck/memory/process[path != 'kernel_task' and size/@units = 'GB' and size &gt; 2]"/>
      
    <xsl:choose>
      <xsl:when
        test="$kernel_task/size[@units = 'GB'] &gt; ($report/etrecheck/hardware/ram * 0.2)">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when test="count($other) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Check for heavy network usage. -->
  <xsl:template name="checkheavynetwork">
  
    <xsl:variable
      name="heavynetwork"
      select="$report/etrecheck/networkusage/process[inputsize/@units = 'GB' or outputsize/@units = 'GB']"/>
      
    <xsl:choose>
      <xsl:when
        test="count($heavynetwork) &gt;= 1 and $report/etrecheck/systemsoftware/uptime &lt; 24">
        <xsl:value-of select="true()"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>

    </xsl:choose>
    
  </xsl:template>

  <!-- Check for out of RAM conditions and a mechanical hard drive. -->
  <xsl:template name="checkoutofram">
  
    <xsl:variable
      name="killedlaunchd"
      select="$report/etrecheck/systemlaunchagents/launchdfile[status = 'killed'] | $report/etrecheck/systemlaunchdaemons/launchdfile[status = 'killed']"/>
      
    <xsl:variable
      name="bootdisk" select="$report/etrecheck/disk/drive[boot = 'true']"/>
    
    <xsl:choose>
      <xsl:when
        test="$report/etrecheck/vm/swapused &gt; 4294967296">
        <xsl:value-of select="true()"/>
      </xsl:when>

      <xsl:when
        test="$bootdisk[mechanism = 'mechanical'] and $report/etrecheck/vm/swapused &gt; 1073741824">
        <xsl:value-of select="true()"/>
      </xsl:when>

      <xsl:when
        test="$bootdisk[mechanism = 'solidstate']">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when
        test="$bootdisk[mechanism = 'mechanical'] and count($killedlaunchd) &gt; 0">
        <xsl:value-of select="true()"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>

    </xsl:choose>

  </xsl:template>

  <!-- Check for an old machine with a new OS. -->
  <xsl:template name="checkoldmachinenewos">
  
    <xsl:variable
      name="newos"
      select="$report/etrecheck/systemsoftware/majorversion &gt; 9"/>
      
    <xsl:choose>
      <xsl:when test="$report/etrecheck/hardware/modeltype = 'MacBook'">
        <xsl:choose>
          <xsl:when
            test="$report/etrecheck/hardware/modelmajorversion &lt; 5">
            <xsl:value-of select="true()"/>
          </xsl:when>
          
          <xsl:otherwise>
            <xsl:value-of select="false()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      
      <xsl:when test="$report/etrecheck/hardware/modeltype = 'MacBookAir'">
        <xsl:choose>
          <xsl:when
            test="$report/etrecheck/hardware/modelmajorversion &lt; 3">
            <xsl:value-of select="true()"/>
          </xsl:when>
          
          <xsl:otherwise>
            <xsl:value-of select="false()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>

      <xsl:when test="$report/etrecheck/hardware/modeltype = 'MacBookPro'">
        <xsl:choose>
          <xsl:when
            test="$report/etrecheck/hardware/modelmajorversion &lt; 5">
            <xsl:value-of select="true()"/>
          </xsl:when>
          
          <xsl:otherwise>
            <xsl:value-of select="false()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>

      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
      
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for Time Machine drive too small. -->
  <xsl:template name="checktmtoosmall">
  
    <xsl:variable name="volumesizes">
      <xsl:for-each select="$report/etrecheck/timemachine//volume">
        <xsl:variable name="value">
          <xsl:call-template name="expand">
            <xsl:with-param name="number" select="used"/>
          </xsl:call-template>
        </xsl:variable>
        <value>
          <xsl:value-of select="$value div 1000 div 1000 div 1000"/>
        </value>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="destinationsizes">
      <xsl:for-each select="$report/etrecheck/timemachine//destination">
        <xsl:variable name="value">
          <xsl:call-template name="expand">
            <xsl:with-param name="number" select="size"/>
          </xsl:call-template>
        </xsl:variable>
        
        <value>
          <xsl:value-of select="$value div 1000 div 1000 div 1000"/>
        </value>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable
      name="maxvolumesize"
      select="math:max(exslt:node-set($volumesizes)/value)"/>
    
    <xsl:variable
      name="maxdestinationsize"
      select="math:max(exslt:node-set($destinationsizes)/value)"/>
    
    <xsl:choose>
      <xsl:when test="$report/etrecheck/timemachine/notconfigured = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      
      <xsl:when
        test="sum($report/etrecheck/timemachine/destinations//count) = 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when test="$maxdestinationsize = 0">
        <xsl:value-of select="false()"/>
      </xsl:when>

      <xsl:when test="$maxdestinationsize &lt; ($maxvolumesize * 2)">
        <xsl:value-of select="true()"/>
      </xsl:when>
    
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
      
    </xsl:choose>

  </xsl:template>
  
  <!-- Check for Unsigned files that look safe. -->
  <xsl:template name="checkminorunsigned">
  
    <xsl:variable
      name="unsafefilecount"
      select="count($report/etrecheck/unsigned//launchdfile[apple != 'true' and (not(details) or (details != 'whitelist' and details != 'whitelistprefix' and details != 'signatureoverride' and details != 'legitimate' and details != 'legitimate_prefix' and details != 'executablemaspermissions' and details != 'plistpermissions' and details != 'executablepermissions'))])"/>
    
    <xsl:variable
      name="launchdfilecount"
      select="count($report/etrecheck/unsigned//launchdfile[apple != 'true'])"/>
      
    <xsl:variable
      name="launchdtaskcount"
      select="count($report/etrecheck/unsigned//launchdtask[apple != 'true'])"/>

    <xsl:variable
      name="loginitemcount"
      select="count($report/etrecheck/unsigned/loginitems/loginitem)"/>

    <xsl:variable
      name="appcount"
      select="count($report/etrecheck/unsigned/applications/application)"/>

    <xsl:variable
      name="unsignedcount"
      select="$launchdfilecount + $launchdtaskcount + $loginitemcount + $appcount"/>

    <xsl:choose>
      <xsl:when test="$unsignedcount = 0">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$unsafefilecount &gt; 0">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$hideunsignedfiles = 'true'">
        <xsl:value-of select="$launchdtaskcount &gt; 0 or $appcount &gt; 0"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="true()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for USB 3.0 drives with a USB 2.0 hub. -->
  <xsl:template name="checkusb2hub">
  
    <xsl:variable
      name="usbdrives"
      select="$report/etrecheck/usb//node[count(drives/device) &gt; 0]"/>
    
    <xsl:variable
      name="usb3drives"
      select="$usbdrives[contains(name, 'USB 3')]"/>
      
    <xsl:variable
      name="usb2hubdrives"
      select="$usb3drives[ancestor::node[contains(name, 'USB2')]]"/>

    <xsl:variable
      name="usb3hubdrives"
      select="$usb2hubdrives[ancestor::node[contains(name, 'USB3')]]"/>

    <xsl:value-of select="count($usb3hubdrives) &gt; 0"/>
    
  </xsl:template>

  <!-- Check for poor performance due to external drives. -->
  <xsl:template name="checkexternaldriveperformance">
  
    <xsl:variable
      name="performanceproblem"
      select="$report/etrecheck/performance/score != 'excellentperformance'"/>

    <xsl:variable
      name="externaldrives"
      select="$report/etrecheck/disk/drive[internal = 'false' and mechanism != 'solidstate']"/>
    
    <xsl:value-of
      select="$performanceproblem = 'true' and count($externaldrives) &gt; 0"/>
    
  </xsl:template>

  <!-- Check for below average performance. -->
  <xsl:template name="checkhddslow">
  
    <xsl:variable name="drivefailure">
      <xsl:call-template name="checkdrivefailure"/>
    </xsl:variable>

    <xsl:variable name="mitigating">
      <xsl:call-template name="checkmitigating"/>
    </xsl:variable>

    <xsl:variable
      name="internalhdd"
      select="$report/etrecheck/disk/drive[internal = 'true' and mechanism = 'mechanical']"/>

    <xsl:variable
      name="slowhddreadspeed"
      select="$report/etrecheck/performance/readspeed &lt; 50"/>
      
    <xsl:variable
      name="slowhddwritespeed"
      select="$report/etrecheck/performance/writespeed &lt; 50"/>

    <xsl:variable
      name="lowfilesystemspeed"
      select="$report/etrecheck/performance/filesystemtime &gt; 100"/>

    <xsl:choose>
      <xsl:when test="$drivefailure = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$mitigating = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$internalhdd and ($slowhddreadspeed or $slowhddwritespeed or $lowfilesystemspeed)">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Check for potential SSD failure. -->
  <xsl:template name="checkssdfailure">
  
    <xsl:variable name="drivefailure">
      <xsl:call-template name="checkdrivefailure"/>
    </xsl:variable>

    <xsl:variable name="mitigating">
      <xsl:call-template name="checkmitigating"/>
    </xsl:variable>

    <xsl:variable name="fusion">
      <xsl:call-template name="checkfusion">
        <xsl:with-param
          name="disk" select="$report/etrecheck/disk[drive/boot = 'true']"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable
      name="internalssd"
      select="$report/etrecheck/disk/drive[internal = 'true' and mechanism = 'solidstate']"/>

    <xsl:variable
      name="slowssdreadspeed"
      select="$report/etrecheck/performance/readspeed &lt; 300"/>
      
    <xsl:variable
      name="slowssdwritespeed"
      select="$report/etrecheck/performance/writespeed &lt; 300"/>

    <xsl:variable
      name="lowfilesystemspeed"
      select="$report/etrecheck/performance/filesystemtime &gt; 100"/>

    <xsl:choose>
      <xsl:when test="$fusion = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$drivefailure = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$mitigating = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$internalssd and ($slowssdreadspeed or $slowssdwritespeed or $lowfilesystemspeed)">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for potential hard drive problem. -->
  <xsl:template name="checkdriveproblem">
  
    <xsl:variable name="drivefailure">
      <xsl:call-template name="checkdrivefailure"/>
    </xsl:variable>

    <xsl:variable name="mitigating">
      <xsl:call-template name="checkmitigating"/>
    </xsl:variable>

    <xsl:variable name="fusion">
      <xsl:call-template name="checkfusion">
        <xsl:with-param
          name="disk" select="$report/etrecheck/disk[drive/boot = 'true']"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable
      name="internalssd"
      select="$report/etrecheck/disk/drive[internal = 'true' and mechanism = 'solidstate']"/>

    <xsl:variable
      name="slowssdreadspeed"
      select="$report/etrecheck/performance/readspeed &lt; 300"/>
      
    <xsl:variable
      name="slowssdwritespeed"
      select="$report/etrecheck/performance/writespeed &lt; 300"/>

    <xsl:variable
      name="lowfilesystemspeed"
      select="$report/etrecheck/performance/filesystemtime &gt; 70"/>

    <xsl:choose>
      <xsl:when test="$mitigating = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$internalssd and ($slowssdreadspeed or $slowssdwritespeed or $lowfilesystemspeed)">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for potential SSD failure. -->
  <xsl:template name="checkfusion">
    <xsl:param name="disk"/>
    
    <xsl:variable
      name="internalssd"
      select="$disk/drive[internal = 'true' and mechanism = 'solidstate']"/>

    <xsl:variable
      name="internalhdd"
      select="$disk/drive[internal = 'true' and mechanism = 'mechanical']"/>

    <xsl:choose>
      <xsl:when test="$internalssd and $internalhdd">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>
  
  <!-- Check for poor performance. -->
  <xsl:template name="checkpoorperformance">
  
    <xsl:variable name="drivefailure">
      <xsl:call-template name="checkdrivefailure"/>
    </xsl:variable>

    <xsl:variable name="mitigating">
      <xsl:call-template name="checkmitigating"/>
    </xsl:variable>

    <xsl:variable name="slowhdd">
      <xsl:call-template name="checkhddslow"/>
    </xsl:variable>

    <xsl:variable name="slowssd">
      <xsl:call-template name="checkssdfailure"/>
    </xsl:variable>

    <xsl:variable name="driveproblem">
      <xsl:call-template name="checkdriveproblem"/>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$drivefailure = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$mitigating = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$slowhdd = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$slowssd = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of
          select="$report/etrecheck/performance/score = 'poorperformance'"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for below average performance. -->
  <xsl:template name="checkbelowaverageperformance">
  
    <xsl:variable name="drivefailure">
      <xsl:call-template name="checkdrivefailure"/>
    </xsl:variable>

    <xsl:variable name="mitigating">
      <xsl:call-template name="checkmitigating"/>
    </xsl:variable>

    <xsl:variable name="slowhdd">
      <xsl:call-template name="checkhddslow"/>
    </xsl:variable>

    <xsl:variable name="slowssd">
      <xsl:call-template name="checkssdfailure"/>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$drivefailure = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$mitigating = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$slowhdd = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$slowssd = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of
          select="$report/etrecheck/performance/score = 'belowaverageperformance'"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for absolute hard drive failure. -->
  <xsl:template name="checkdrivefailure">
  
    <xsl:variable
      name="drivefailure"
      select="sum($report/etrecheck/diagnostics/event[drivefailure = 'true']/count) &gt; 0"/>

    <xsl:variable name="mitigating">
      <xsl:call-template name="checkmitigating"/>
    </xsl:variable>

    <xsl:variable
      name="failreadspeed"
      select="$report/etrecheck/performance/readspeed &lt; 30"/>
      
    <xsl:variable
      name="failwritespeed"
      select="$report/etrecheck/performance/writespeed &lt; 30"/>
      
    <xsl:variable
      name="lowfilesystemspeed"
      select="$report/etrecheck/performance/filesystemtime &gt; 100"/>

    <xsl:variable
      name="slowreadspeed"
      select="$report/etrecheck/performance/readspeed &lt; 50"/>
      
    <xsl:variable
      name="slowwritespeed"
      select="$report/etrecheck/performance/writespeed &lt; 50"/>

    <xsl:choose>
      <xsl:when test="$drivefailure = 'true'">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when test="$mitigating = 'true'">
        <xsl:value-of select="false()"/>
      </xsl:when>
      <xsl:when test="$failreadspeed or $failwritespeed">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when test="$lowfilesystemspeed and ($slowreadspeed or $slowwritespeed)">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for mitigating circumstances. -->
  <xsl:template name="checkmitigating">
  
    <!-- Encryption in progress. -->
    <xsl:variable
      name="encrypting"
      select="$report/etrecheck//volume/encryption/status = 'encrypting'"/>
  
    <!-- Decryption in progress. -->
    <xsl:variable
      name="decrypting"
      select="$report/etrecheck//volume/encryption/status = 'decrypting'"/>
  
    <!-- Encryption or decryption in progress. -->
    <xsl:variable
      name="inprogress"
      select="$report/etrecheck//volume/encryption/status = 'inprogress'"/>

    <xsl:choose>
      <xsl:when test="$encrypting">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when test="$decrypting">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when test="$inprogress">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

</xsl:stylesheet>
